from RSTAB.initModel import Model, ConvertStrToListOfInt, GetAllAddonStatuses
from RSTAB.enums import ObjectTypes
from suds.sax.text import Text
import sys

class GetObjectNumbersByType():

    def __new__(cls,
                ObjectType = ObjectTypes.E_OBJECT_TYPE_NODE,
                model = Model):

        """
        Returns a sorted list which contains object numbers in RSTAB tables.
        ObjectNumberList = [1, 2, ... ] or [[1,2], [2,2]]

        Args:
            ObjectType (enum): Object type enum
            model (RSTAB Class, optional): Model to be edited
        Returns:
            ObjectNumberList (list): Sorted list of object numbers or list of lists if there is parent number (e.g. nodal_load).
        """

        ObjectNumber = model.clientModel.service.get_all_object_numbers_by_type(ObjectType.name)
        ObjectNumberList = []

        if len(ObjectNumber):
            for i in range(len(ObjectNumber.item)):
                # this is used when requesting objects in loads (E_OBJECT_TYPE_NODAL_LOAD, E_OBJECT_TYPE_NODAL_LOAD etc.)
                try:
                    children = ConvertStrToListOfInt(ObjectNumber.item[i].children)
                    for c in children:
                        ObjectNumberList.append([c, ObjectNumber.item[i].no])
                # all other objects
                except:
                    ObjectNumberList.append(ObjectNumber.item[i].no)

            if isinstance(ObjectNumberList[0], list):
                ObjectNumberList = sorted(ObjectNumberList, key=lambda x: x[1])
            else:
                ObjectNumberList.sort()

        return ObjectNumberList

class GetAllObjects():
    """
    Returns tuple of 2 lists containing all objects and their parameters and list of imports needed to facilitate re-creating theese.
    Args:
        model(RSTAB Class, optional): Model to be edited
    Returns:
        objects: List of all objects that can be created via Client.
        imports: List of imports needed to be able to create objects from the first list.
    """
    def __new__(cls,
                model = Model):

        # Steps to retrieve data from RSTAB:
        # 1) get numbers of given object type via GetObjectNumbersByType(),
        # 2) get data from individual objects,
        # 3) set import of the type,
        # 4) set individual objects.
        # For each of these steps individual record is made (4 total).

        # Vector of all function
        func_vec = [[ObjectTypes.E_OBJECT_TYPE_COORDINATE_SYSTEM, lambda i: model.clientModel.service.get_coordinate_system(i), 'from RSTAB.BasicObjects.coordinateSystem import CoordinateSystem\n', 'CoordinateSystem'],
			[ObjectTypes.E_OBJECT_TYPE_MATERIAL, lambda i: model.clientModel.service.get_material(i), 'from RSTAB.BasicObjects.material import Material\n', 'Material'],
            [ObjectTypes.E_OBJECT_TYPE_SECTION, lambda i: model.clientModel.service.get_section(i), 'from RSTAB.BasicObjects.section import Section\n', 'Section'],
            [ObjectTypes.E_OBJECT_TYPE_NODE, lambda i: model.clientModel.service.get_node(i), 'from RSTAB.BasicObjects.node import Node\n', 'Node'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER, lambda i: model.clientModel.service.get_member(i), 'from RSTAB.BasicObjects.member import Member\n', 'Member'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_SET, lambda i: model.clientModel.service.get_member_set(i), 'from RSTAB.BasicObjects.memberSet import MemberSet\n', 'MemberSet'],
            [ObjectTypes.E_OBJECT_TYPE_STRUCTURE_MODIFICATION, lambda i: model.clientModel.service.get_structure_modification(i), 'from RSTAB.SpecialObjects.structureModification import StructureModification\n', 'StructureModification'],
            [ObjectTypes.E_OBJECT_TYPE_NODAL_RELEASE, lambda i: model.clientModel.service.get_nodal_release(i), 'from RSTAB.SpecialObjects.nodalRelease import NodalRelease\n', 'NodalRelease'],
            # blocks

            [ObjectTypes.E_OBJECT_TYPE_NODAL_SUPPORT, lambda i: model.clientModel.service.get_nodal_support(i), 'from RSTAB.TypesForNodes.nodalSupport import NodalSupport\n', 'NodalSupport'],

            [ObjectTypes.E_OBJECT_TYPE_MEMBER_HINGE, lambda i: model.clientModel.service.get_member_hinge(i), 'from RSTAB.TypesForMembers.memberHinge import MemberHinge\n', 'MemberHinge'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_ECCENTRICITY, lambda i: model.clientModel.service.get_member_eccentricity(i), 'from RSTAB.TypesForMembers.memberEccentricity import MemberEccentricity\n', 'MemberEccentricity'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_SUPPORT, lambda i: model.clientModel.service.get_member_support(i), 'from RSTAB.TypesForMembers.memberSupport import MemberSupport\n', 'MemberSupport'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_TRANSVERSE_STIFFENER, lambda i: model.clientModel.service.get_member_transverse_stiffeners(i), 'from RSTAB.TypesForMembers.memberTransverseStiffeners import MemberTransverseStiffeners\n', 'MemberTransverseStiffeners'],
            # member_opening
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_STIFFNESS_MODIFICATION, lambda i: model.clientModel.service.get_member_stiffness_modification(i), 'from RSTAB.TypesForMembers.memberStiffnessModification import MemberStiffnessModification\n', 'MemberStiffnessModification'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_NONLINEARITY, lambda i: model.clientModel.service.get_member_nonlinearity(i), 'from RSTAB.TypesForMembers.memberNonlinearity import MemberNonlinearity\n', 'MemberNonlinearity'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_DEFINABLE_STIFFNESS, lambda i: model.clientModel.service.get_member_definable_stiffness(i), 'from RSTAB.TypesForMembers.memberDefinableStiffness import MemberDefinableStiffness\n', 'MemberDefinableStiffness'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_RESULT_INTERMEDIATE_POINT, lambda i: model.clientModel.service.get_member_result_intermediate_point(i), 'from RSTAB.TypesForMembers.memberResultIntermediatePoints import MemberResultIntermediatePoint\n', 'MemberResultIntermediatePoint'],
            # design_support
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_ROTATIONAL_RESTRAINT, lambda i: model.clientModel.service.get_member_rotational_restraint(i), 'from RSTAB.TypesForMembers.memberRotationalRestraint import MemberRotationalRestraint\n', 'MemberRotationalRestraint'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_SHEAR_PANEL, lambda i: model.clientModel.service.get_member_shear_panel(i), 'from RSTAB.TypesForMembers.memberShearPanel import MemberShearPanel\n', 'MemberShearPanel'],

            # nodal_release_type
            [ObjectTypes.E_OBJECT_TYPE_CONCRETE_EFFECTIVE_LENGTHS, lambda i: model.clientModel.service.get_concrete_effective_lengths(i), 'from RSTAB.TypesforConcreteDesign.ConcreteEffectiveLength import ConcreteEffectiveLength\n', 'ConcreteEffectiveLength'],
            [ObjectTypes.E_OBJECT_TYPE_CONCRETE_DURABILITY, lambda i: model.clientModel.service.get_concrete_durability(i), 'from RSTAB.TypesforConcreteDesign.ConcreteDurability import ConcreteDurability\n', 'ConcreteDurability'],
            [ObjectTypes.E_OBJECT_TYPE_CONCRETE_DESIGN_SLS_CONFIGURATION, lambda i: model.clientModel.service.get_concrete_design_sls_configuration(i), 'from RSTAB.ConcreteDesign.ConcreteServiceabilityConfigurations import ConcreteServiceabilityConfiguration\n', 'ConcreteServiceabilityConfiguration'],
            [ObjectTypes.E_OBJECT_TYPE_CONCRETE_DESIGN_ULS_CONFIGURATION, lambda i: model.clientModel.service.get_concrete_design_uls_configuration(i), 'from RSTAB.ConcreteDesign.ConcreteUltimateConfigurations import ConcreteUltimateConfiguration\n', 'ConcreteUltimateConfiguration'],
            [ObjectTypes.E_OBJECT_TYPE_STEEL_EFFECTIVE_LENGTHS, lambda i: model.clientModel.service.get_steel_effective_lengths(i), 'from RSTAB.TypesForSteelDesign.steelEffectiveLengths import SteelEffectiveLengths\n', 'SteelEffectiveLengths'],
            [ObjectTypes.E_OBJECT_TYPE_STEEL_BOUNDARY_CONDITIONS, lambda i: model.clientModel.service.get_steel_boundary_conditions(i), 'from RSTAB.TypesForSteelDesign.steelBoundaryConditions import SteelBoundaryConditions\n', 'SteelBoundaryConditions'],
            [ObjectTypes.E_OBJECT_TYPE_STEEL_MEMBER_LOCAL_SECTION_REDUCTION, lambda i: model.clientModel.service.get_steel_member_local_section_reduction(i), 'from RSTAB.TypesForSteelDesign.SteelMemberLocalSectionReduction import SteelMemberLocalSectionReduction\n', 'SteelMemberLocalSectionReduction'],

            [ObjectTypes.E_OBJECT_TYPE_STEEL_DESIGN_SLS_CONFIGURATION, lambda i: model.clientModel.service.get_steel_design_sls_configuration(i), 'from RSTAB.SteelDesign.steelServiceabilityConfiguration import SteelDesignServiceabilityConfigurations\n', 'SteelDesignServiceabilityConfigurations'],
            [ObjectTypes.E_OBJECT_TYPE_STEEL_DESIGN_ULS_CONFIGURATION, lambda i: model.clientModel.service.get_steel_design_uls_configuration(i), 'from RSTAB.SteelDesign.steelUltimateConfigurations import SteelDesignUltimateConfigurations\n', 'SteelDesignUltimateConfigurations'],
            [ObjectTypes.E_OBJECT_TYPE_TIMBER_EFFECTIVE_LENGTHS, lambda i: model.clientModel.service.get_timber_effective_lengths(i), 'from RSTAB.TypesForTimberDesign.timberEffectiveLengths import TimberEffectiveLengths\n', 'TimberEffectiveLengths'],
            [ObjectTypes.E_OBJECT_TYPE_TIMBER_SERVICE_CLASS, lambda i: model.clientModel.service.get_timber_service_class(i), 'from RSTAB.TypesForTimberDesign.timberServiceClass import TimberServiceClass\n', 'TimberServiceClass'],
            [ObjectTypes.E_OBJECT_TYPE_ALUMINUM_MEMBER_LOCAL_SECTION_REDUCTION, lambda i: model.clientModel.service.get_timber_member_local_section_reduction(i), 'from RSTAB.TypesForTimberDesign.timberMemberLocalSectionReduction import TimberMemberLocalSectionReduction\n', 'TimberMemberLocalSectionReduction'],
            [ObjectTypes.E_OBJECT_TYPE_TIMBER_DESIGN_SLS_CONFIGURATION, lambda i: model.clientModel.service.get_timber_design_sls_configuration(i), 'from RSTAB.TimberDesign.timberServiceLimitStateConfigurations import TimberDesignServiceLimitStateConfigurations\n', 'TimberDesignServiceLimitStateConfigurations'],
            [ObjectTypes.E_OBJECT_TYPE_TIMBER_DESIGN_ULS_CONFIGURATION, lambda i: model.clientModel.service.get_timber_design_uls_configuration(i), 'from RSTAB.TimberDesign.timberUltimateConfigurations import TimberDesignUltimateConfigurations\n', 'TimberDesignUltimateConfigurations'],
            [ObjectTypes.E_OBJECT_TYPE_ALUMINUM_EFFECTIVE_LENGTHS, lambda i: model.clientModel.service.get_aluminum_effective_lengths(i), 'from RSTAB.TypesForAluminumDesign.aluminumEffectiveLengths import AluminumEffectiveLengths\n', 'AluminumEffectiveLengths'],
            [ObjectTypes.E_OBJECT_TYPE_ALUMINUM_MEMBER_LOCAL_SECTION_REDUCTION, lambda i: model.clientModel.service.get_aluminum_member_local_section_reduction(i), 'from RSTAB.TypesForAluminumDesign.aluminumMemberLocalSectionReduction import AluminumMemberLocalSectionReduction\n', 'AluminumMemberLocalSectionReduction'],
            [ObjectTypes.E_OBJECT_TYPE_ALUMINUM_MEMBER_TRANSVERSE_WELD, lambda i: model.clientModel.service.get_aluminum_member_transverse_weld(i), 'from RSTAB.TypesForAluminumDesign.aluminumMemberTransverseWelds import AluminumMemberTransverseWeld\n', 'AluminumMemberTransverseWeld'],
            # steel_joints
            # craneways
            # cran
            [ObjectTypes.E_OBJECT_TYPE_IMPERFECTION_CASE, lambda i: model.clientModel.service.get_imperfection_case(i), 'from RSTAB.Imperfections.imperfectionCase import ImperfectionCase\n', 'ImperfectionCase'],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_IMPERFECTION, lambda i,j: model.clientModel.service.get_member_imperfection(i,j), 'from RSTAB.Imperfections.memberImperfection import MemberImperfection\n', 'MemberImperfection(imperfection_case='],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_SET_IMPERFECTION, lambda i: model.clientModel.service.get_member_set_imperfection(i), 'from RSTAB.Imperfections.membersetImperfection import MemberSetImperfection\n', 'MemberSetImperfection(imperfection_case='],
            # construction_stag
            [ObjectTypes.E_OBJECT_TYPE_LOAD_CASE, lambda i: model.clientModel.service.get_load_case(i), 'from RSTAB.LoadCasesAndCombinations.loadCase import LoadCase\n', 'LoadCase'],
            [ObjectTypes.E_OBJECT_TYPE_DESIGN_SITUATION, lambda i: model.clientModel.service.get_design_situation(i), 'from RSTAB.LoadCasesAndCombinations.designSituation import DesignSituation\n', 'DesignSituation'],
            [ObjectTypes.E_OBJECT_TYPE_LOAD_COMBINATION, lambda i: model.clientModel.service.get_load_combination(i), 'from RSTAB.LoadCasesAndCombinations.loadCombination import LoadCombination\n', 'LoadCombination'],
            #[ObjectTypes, lambda i: model.clientModel.service.get_(i), 'from RSTAB.LoadCasesAndCombinations.loadCasesAndCombinations import LoadCasesAndCombinations',]
            [ObjectTypes.E_OBJECT_TYPE_RESULT_COMBINATION, lambda i: model.clientModel.service.get_result_combination(i), 'from RSTAB.LoadCasesAndCombinations.resultCombination import ResultCombination\n', 'ResultCombination'],
            [ObjectTypes.E_OBJECT_TYPE_STATIC_ANALYSIS_SETTINGS, lambda i: model.clientModel.service.get_static_analysis_settings(i), 'from RSTAB.LoadCasesAndCombinations.staticAnalysisSettings import StaticAnalysisSettings\n', 'StaticAnalysisSettings'],
            [ObjectTypes.E_OBJECT_TYPE_STABILITY_ANALYSIS_SETTINGS, lambda i: model.clientModel.service.get_stability_analysis_settings(i), 'from RSTAB.LoadCasesAndCombinations.stabilityAnalysisSettings import StabilityAnalysisSettings\n', 'StabilityAnalysisSettings'],
            [ObjectTypes.E_OBJECT_TYPE_MODAL_ANALYSIS_SETTINGS, lambda i: model.clientModel.service.get_modal_analysis_settings(i), 'from RSTAB.LoadCasesAndCombinations.modalAnalysisSettings import ModalAnalysisSettings\n', 'ModalAnalysisSettings'],
            [ObjectTypes.E_OBJECT_TYPE_SPECTRAL_ANALYSIS_SETTINGS, lambda i: model.clientModel.service.get_spectral_analysis_settings(i), 'from RSTAB.LoadCasesAndCombinations.spectralAnalysisSettings import SpectralAnalysisSettings\n', 'SpectralAnalysisSettings'],
            [ObjectTypes.E_OBJECT_TYPE_COMBINATION_WIZARD, lambda i: model.clientModel.service.get_combination_wizard(i), 'from RSTAB.LoadCasesAndCombinations.combinationWizard import CombinationWizard\n', 'CombinationWizard'],
            # member_loads_from_area_loads
            # member_loads_from_free_line_load
            # snow_loads
            # wind_loads
            # wind_profiles
            # wind_simulatio
            [ObjectTypes.E_OBJECT_TYPE_NODAL_LOAD, lambda i,j: model.clientModel.service.get_nodal_load(i,j), 'from RSTAB.Loads.nodalLoad import NodalLoad\n', 'NodalLoad(load_case_no='],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_LOAD, lambda i,j: model.clientModel.service.get_member_load(i,j), 'from RSTAB.Loads.memberLoad import MemberLoad\n', 'MemberLoad(load_case_no='],
            [ObjectTypes.E_OBJECT_TYPE_MEMBER_SET_LOAD, lambda i,j: model.clientModel.service.get_member_set_load(i,j), 'from RSTAB.Loads.membersetload import MemberSetLoad\n', 'MemberSetLoad(load_case_no='],
            [ObjectTypes.E_OBJECT_TYPE_IMPOSED_NODAL_DEFORMATION, lambda i,j: model.clientModel.service.get_imposed_nodal_deformation(i,j), 'from RSTAB.Loads.imposedNodalDeformation import ImposedNodalDeformation\n', 'ImposedNodalDeformation(load_case_no=']]
            # calculations_diagram
            #[ObjectTypes.E_OBJECT_TYPE_RESPONSE_SPECTRUM, lambda i: model.clientModel.service.get_response_spectrum(i), 'from RSTAB.DynamicLoads.responseSpectrum import ResponseSpectrum\n', 'ResponseSpectrum']]

        def convertSubclases(param):
            '''
            Convert structures/parameters to approriate structure
            '''
            if isinstance(param, str) or isinstance(param, int) or isinstance(param, float) or isinstance(param, bool):
                pass
            elif isinstance(param, list):
                toDel = []
                for i,j in enumerate(param):
                    # Parameters of value None, UNKNOWN or "" are not exported
                    if j == None or j == 'UNKNOWN' or j == "":
                        toDel.append(i)
                    elif isinstance(j, Text):
                        # Change Text to string
                        param[i] = str(j)
                    else:
                        param[i] = convertSubclases(j)
                for td in toDel:
                    param.remove(td)
            else:
                param = dict(param)
                toDel = []
                for key in param.keys().__reversed__():
                    # Parameters of value None, UNKNOWN or "" are not exported
                    if param[key] == None or param[key] == 'UNKNOWN' or param[key] == "":
                        toDel.append(key)
                    elif isinstance(param[key], Text):
                        # Change Text to string
                        param[key] = str(param[key])
                    else:
                        param[key] = convertSubclases(param[key])
                for td in toDel:
                    del param[td]
            return param

        imports = [] # defines what to import with 'imports'
        objects = [] # individual lines written in file

        # Load Cases and Combinations setup
        loadCasesAndCombinations = convertSubclases(dict(model.clientModel.service.get_load_cases_and_combinations()))
        settingsAndOptions = dict(model.clientModel.service.get_model_settings_and_options())
        addonStatuses = GetAllAddonStatuses(model.clientModel)
        del settingsAndOptions['date_of_zero_day']
        settingsAndOptions = convertSubclases(settingsAndOptions)

        objects.append('LoadCasesAndCombinations(params='+str(loadCasesAndCombinations)+')\n')
        objects.append('BaseSettings(params='+str(settingsAndOptions)+')\n')
        objects.append('SetAddonStatuses('+str(addonStatuses)+')\n')
        imports.append('from RSTAB.LoadCasesAndCombinations.loadCasesAndCombinations import LoadCasesAndCombinations\n')
        imports.append('from RSTAB.baseSettings import BaseSettings, MainObjectsToActivate\n')

        # Get number of every type of object supported by Client.
        # Get info of each existing object.
        # Add import to lines.
        # Add each object to lines.
        for id, func in enumerate(func_vec):

            objNumbers = GetObjectNumbersByType(ObjectType=func[0])
            if objNumbers:
                addImport = False
                for idx,i in enumerate(objNumbers):

                    # Print status
                    percent = (idx/len(objNumbers))*100
                    sys.stdout.write("\r                                                                       ")
                    sys.stdout.write("\r{}: {:.0f}%, total progress: {:.1f}% ".format(func[3], (id/len(func_vec))*100, percent))
                    sys.stdout.flush()

                    try:
                        if isinstance(i, list):
                            params = dict(func[1](i[0], i[1]))
                        else:
                            params = dict(func[1](i))
                        addImport = True
                    except:
                        print('INFO: There seems to a be phantom object or issue in your model. Type:', func[3], ', number:', str(i))
                        continue
                    params = convertSubclases(params)
                    # don't set this parameter
                    del params['id_for_export_import']

                    if isinstance(i, list):
                        objects.append(func[3]+str(i[1])+', params='+str(params)+')\n')
                    else:
                        objects.append(func[3]+'(params='+str(params)+')\n')

                # Add import if at least one object was added
                if addImport:
                    imports.append(func[2])

        sys.stdout.write("\r                                                                       ")
        sys.stdout.write("\rDone 100%\n")
        sys.stdout.flush()
        return (objects, imports)



from RSTAB.initModel import clearAttributes, deleteEmptyAttributes, Model, ConvertStrToListOfInt
from RSTAB.enums import ObjectTypes, MaterialType, MaterialModel, MaterialDefinitionType
from RSTAB.enums import MaterialStiffnessModificationType, PoissonRatioEditableGroupType

class Material():
    def __init__(self,
                 no: int = 1,
                 name: str = 'S235',
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Material Tag
            name (str): Name of Desired Material (As Named in RSTAB Database)
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Material
        clientObject = model.clientModel.factory.create('ns0:material')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Material No.
        clientObject.no = no

        # Material Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add material to client model
        model.clientModel.service.set_material(clientObject)

    @staticmethod
    def UserDefinedMaterial(no: int = 1,
                            name: str = 'S235',
                            material_type = MaterialType.TYPE_CONCRETE,
                            material_model = MaterialModel.MODEL_ISOTROPIC_LINEAR_ELASTIC,
                            elasticity_modulus: float = None,
                            elasticity_modulus_x: float = None,
                            elasticity_modulus_y: float = None,
                            elasticity_modulus_z: float = None,
                            shear_modulus: float = None,
                            shear_modulus_yz: float = None,
                            shear_modulus_xz: float = None,
                            shear_modulus_xy: float = None,
                            poisson_ratio: float = None,
                            poisson_ratio_yz: float = None,
                            poisson_ratio_xz: float = None,
                            poisson_ratio_xy: float = None,
                            poisson_ratio_zy: float = None,
                            poisson_ratio_zx: float = None,
                            poisson_ratio_yx: float = None,
                            poisson_ratio_editable_group_type = PoissonRatioEditableGroupType.POISSON_RATIOS_GROUP_MAJOR_3D,
                            mass_density: float = None,
                            thermal_expansion_coefficient: float = None,
                            thermal_expansion_coefficient_x: float = None,
                            thermal_expansion_coefficient_y: float = None,
                            thermal_expansion_coefficient_z: float = None,
                            definition_type = MaterialDefinitionType.E_G_NU,
                            stiffness_modification_type = MaterialStiffnessModificationType.STIFFNESS_MODIFICATION_TYPE_DIVISION,
                            division_multiplication_factor: float = 1,
                            comment: str = '',
                            params: dict = None,
                            model = Model):

        '''
        Args:
            no (int): Material Tag
            name (str): User Defined Material Name
            material_type (enum): Material Type Enumeration
            material_model (enum): Material Model Enumeration
            elasticity_modulus (float): Elasticity Modulus
            elasticity_modulus_x (float): X-Direction Elasticity Modulus
            elasticity_modulus_y (float): Y-Direction Elasticity Modulus
            elasticity_modulus_z (float): Z-Direction Elasticity Modulus
            shear_modulus (float): Shear Modulus
            shear_modulus_yz (float): YZ-Direction Shear Modulus
            shear_modulus_xz (float): XZ-Direction Shear Modulus
            shear_modulus_xy (float): XY-Direction Shear Modulus
            poisson_ratio (float): Poisson Ratio
            poisson_ratio_yz (float): YZ-Direction Poisson Ratio
            poisson_ratio_xz (float): XZ-Direction Poisson Ratio
            poisson_ratio_xy (float): XY-Direction Poisson Ratio
            poisson_ratio_zy (float): ZY-Direction Poisson Ratio
            poisson_ratio_zx (float): ZX-Direction Poisson Ratio
            poisson_ratio_yx (float): YX-Direction Poisson Ratio
            poisson_ratio_editable_group_type (enum): Poisson Ratio Editable Group Type Enumeration
            mass_density (float): Mass Density
            thermal_expansion_coefficient (float): Thermal Expansion Coefficient
            thermal_expansion_coefficient_x (float): X-Direction Thermal Expansion Coefficient
            thermal_expansion_coefficient_y (float): Y-Direction Thermal Expansion Coefficient
            thermal_expansion_coefficient_z (float): Z-Direction Thermal Expansion Coefficient
            definition_type (enum): Material Definition Type Enumeration
            stiffness_modification_type (enum): Material Stiffness Modification Type Enumeration
            division_multiplication_factor (float): Division Multiplication Factor
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Material
        clientObject = model.clientModel.factory.create('ns0:material')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Material No.
        clientObject.no = no

        clientObject.user_defined = True

        # Material Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Material Type
        clientObject.material_type = material_type.name

        # Material Model
        clientObject.material_model = material_model.name

        # Basic Material Properties
        clientObject.temperature = model.clientModel.factory.create('ns0:material.temperature')

        mbp = model.clientModel.factory.create('ns0:material_temperature_row')
        mbp.no = 1
        mbp.row = model.clientModel.factory.create('ns0:material_temperature')
        clearAttributes(mbp.row)
        mbp.row.poisson_ratio_editable_group_type = poisson_ratio_editable_group_type.name
        mbp.row.elasticity_modulus_global = elasticity_modulus
        mbp.row.elasticity_modulus_y = elasticity_modulus_y
        mbp.row.elasticity_modulus_x = elasticity_modulus_x
        mbp.row.elasticity_modulus_z = elasticity_modulus_z
        mbp.row.shear_modulus_global = shear_modulus
        mbp.row.shear_modulus_yz = shear_modulus_yz
        mbp.row.shear_modulus_xz = shear_modulus_xz
        mbp.row.shear_modulus_xy = shear_modulus_xy
        mbp.row.poisson_ratio_global = poisson_ratio
        mbp.row.poisson_ratio_yz = poisson_ratio_yz
        mbp.row.poisson_ratio_xz = poisson_ratio_xz
        mbp.row.poisson_ratio_xy = poisson_ratio_xy
        mbp.row.poisson_ratio_zy = poisson_ratio_zy
        mbp.row.poisson_ratio_zx = poisson_ratio_zx
        mbp.row.poisson_ratio_yx = poisson_ratio_yx
        mbp.row.mass_density = mass_density
        mbp.row.thermal_expansion_coefficient_global = thermal_expansion_coefficient
        mbp.row.thermal_expansion_coefficient_x = thermal_expansion_coefficient_x
        mbp.row.thermal_expansion_coefficient_y = thermal_expansion_coefficient_y
        mbp.row.thermal_expansion_coefficient_z = thermal_expansion_coefficient_z
        mbp.row.division_multiplication_factor = division_multiplication_factor

        clientObject.temperature.material_temperature.append(mbp)

        # Definition Type
        clientObject.definition_type = definition_type.name

        # Stiffness Modification
        if stiffness_modification_type:
            clientObject.stiffness_modification = True
            clientObject.stiffness_modification_type = stiffness_modification_type.name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add material to client model
        model.clientModel.service.set_material(clientObject)

    @staticmethod
    def DeleteMaterial(materials_no: str = '1 2', model = Model):

        '''
        Args:
            materials_no (str): Numbers of Materials to be deleted
            model (RSTAB Class, optional): Model to be edited
        '''

        # Delete from client model
        for material in ConvertStrToListOfInt(materials_no):
            model.clientModel.service.delete_object(ObjectTypes.E_OBJECT_TYPE_MATERIAL.name, material)

    @staticmethod
    def GetMaterial(object_index: int = 1, model = Model):

        '''
        Args:
            obejct_index (int): Material Index
            model (RSTAB Class, optional): Model to be edited
        '''

        # Get Material from client model
        return model.clientModel.service.get_material(object_index)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes,ConvertToDlString
from RSTAB.enums import MemberSpringType, PartialActivityAlongType, MemberSpringSelfWeightDefinition

class MemberSpring():

    def __init__(self,
                 no: int = 1,
                 member: str = "",
                 definition_type = MemberSpringType.PARTIAL_ACTIVITY,
                 parameters: list = [[PartialActivityAlongType.PARTIAL_ACTIVITY_TYPE_COMPLETE, 0.0], [PartialActivityAlongType.PARTIAL_ACTIVITY_TYPE_COMPLETE, 0.0]],
                 axial_stiffness: float = 0.0,
                 self_weight: list = [MemberSpringSelfWeightDefinition.MASS_PER_LENGTH, 1],
                 name: str = None,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Member Spring Tag
            members (str): Assigned Members
            definition_type (enum): Member Spring Type Enumeration
            parameters (list of lists): List of Parameters for Spring Nonlinearity
                for definition_type == 'PARTIAL_ACTIVITY':
                    parameters = [negative zone, positive zone]
                    for negative/positive zone[0] == PartialActivityAlongType.PARTIAL_ACTIVITY_TYPE_COMPLETE:
                        negative/positive zone = [negative/positive zone type (enum), slippage (float)]
                    for negative/positive zone[0] == PartialActivityAlongType.PARTIAL_ACTIVITY_TYPE_FIXED:
                        negative/positive zone = [negative/positive zone type (enum), slippage (float), displacement (float)]  (Note: Displacement must be greater than slippage)
                    for negative/positive zone[0] == PartialActivityAlongType.PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE/PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE:
                        negative/positive zone = [negative/positive zone type (enum), slippage (float), force (float)]
                for definition_type == 'DIAGRAM':
                    parameters = [[symmetric(bool), LineReleaseDiagram Enumeration(start), LineReleaseDiagram Enumeration(end)], [[displacement, force],...]]
            axial_stiffness (float): Axial Stiffness
            self_weight (list): Self Weight Parameters List
                for self_weight[0] == 'MASS':
                    self_weight = [MemberSpringSelfWeightDefinition.MASS, mass (float)]
                for self_weight[0] == 'MASS_PER_LENGTH':
                    self_weight = [MemberSpringSelfWeightDefinition.MASS_PER_LENGTH, mass_per_length (float)]
                for self_weight[0] == 'SPECIFIC_WEIGHT':
                    self_weight = [MemberSpringSelfWeightDefinition.SPECIFIC_WEIGHT, specific_weight (float), section_area (float)]
            name (str, option): User Defined Design Support Name
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Member Hinge
        clientObject = model.clientModel.factory.create('ns0:member_spring')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Spring No.
        clientObject.no = no

        # Assigned Member
        clientObject.assigned_to = ConvertToDlString(member)

        # Member Spring Type
        clientObject.definition_type = definition_type.name

        # Spring Nonlinearity
        if definition_type.name == 'PARTIAL_ACTIVITY':

            # Negative Zone
            clientObject.partial_activity_along_x_negative_type = parameters[0][0].name

            if parameters[0][0].name == "PARTIAL_ACTIVITY_TYPE_COMPLETE":
                clientObject.partial_activity_along_x_negative_slippage = parameters[0][1]

            elif parameters[0][0].name == "PARTIAL_ACTIVITY_TYPE_FIXED":
                clientObject.partial_activity_along_x_negative_slippage = parameters[0][1]
                clientObject.partial_activity_along_x_negative_displacement = parameters[0][2]

            elif parameters[0][0].name == ("PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE" or "PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE"):
                clientObject.partial_activity_along_x_negative_slippage = parameters[0][1]
                clientObject.partial_activity_along_x_negative_force = parameters[0][2]

            # Positive Zone
            clientObject.partial_activity_along_x_positive_type = parameters[1][0].name

            if parameters[1][0].name == "PARTIAL_ACTIVITY_TYPE_COMPLETE":
                clientObject.partial_activity_along_x_positive_slippage = parameters[1][1]

            elif parameters[1][0].name == "PARTIAL_ACTIVITY_TYPE_FIXED":
                clientObject.partial_activity_along_x_positive_slippage = parameters[1][1]
                clientObject.partial_activity_along_x_positive_displacement = parameters[1][1]

            elif parameters[1][0].name == ("PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE" or "PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE"):
                clientObject.partial_activity_along_x_positive_slippage = parameters[1][1]
                clientObject.partial_activity_along_x_positive_force = parameters[1][2]

        elif definition_type.name == 'DIAGRAM':

            clientObject.diagram_along_x_symmetric = parameters[0][0]
            clientObject.diagram_along_x_is_sorted = True

            if parameters[0][0]:
                clientObject.diagram_along_x_start = parameters[0][1].name
                clientObject.diagram_along_x_end = parameters[0][1].name

            else:
                clientObject.diagram_along_x_start = parameters[0][1].name
                clientObject.diagram_along_x_end = parameters[0][2].name

            clientObject.diagram_along_x_table = model.clientModel.factory.create('ns0:member_spring.diagram_along_x_table')

            for i,j in enumerate(parameters[1]):
                msdx = model.clientModel.factory.create('ns0:member_spring_diagram_along_x_table_row')
                msdx.no = i+1
                msdx.row = model.clientModel.factory.create('ns0:member_spring_diagram_along_x_table')
                clearAttributes(msdx.row)
                msdx.row.displacement = parameters[1][i][0]
                msdx.row.force = parameters[1][i][1]

                clientObject.diagram_along_x_table.member_spring_diagram_along_x_table.append(msdx)

        # Axial Stiffness
        clientObject.axial_stiffness = axial_stiffness

        # Self Weight
        clientObject.self_weight_definition = self_weight[0].name

        if self_weight[0].name == 'MASS':
            clientObject.mass = self_weight[1]

        elif self_weight[0].name == 'MASS_PER_LENGTH':
            clientObject.mass_per_length = self_weight[1]

        elif self_weight[0].name == 'SPECIFIC_WEIGHT':
            clientObject.specific_weight = self_weight[1]
            clientObject.section_area = self_weight[2]

        # User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Member Spring to client model
        model.clientModel.service.set_member_spring(clientObject)




from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.enums import DurabilityStructuralClassType, DurabilityAllowanceDeviationType

class ConcreteDurability():
    def __init__(self,
                no: int = 1,
                name: str = "XC 1",
                members_no: str = "1",
                member_sets_no: str = "1",
                exposure_classes_reinforcement: list = [True, False, False, False],
                exposure_classes_reinforcement_types: list = None,
                exposure_classes_concrete: list = [False, False, False],
                exposure_classes_concrete_types: list = None,
                structural_class: list = [DurabilityStructuralClassType.STANDARD, False, False, False, False],
                stainless_steel_reduction: list = [False],
                additional_protection_reduction: list = [False],
                allowance_deviation: list = [DurabilityAllowanceDeviationType.STANDARD, False],
                comment: str = '',
                params: dict = None,
                model = Model):
        """
        Args:
            no (int): Concrete Durability Tag
            name (str): User Defined Name
            members_no (str): Assigned Members
            member_sets_no (str): Assigned Member Sets
            exposure_classes_reinforcement (list): Exposure Classes Reinforcement Parameters
            exposure_classes_reinforcement_types (list of enum): Exposure Classes Reinforcement Type List of Enumeration
            exposure_classes_concrete (list): Exposure Classes Concrete Parameters
            exposure_classes_concrete_types (list of enum): Exposure Classes Concrete Type List of Enumeration
            structural_class (list): Structural Class Parameters
            stainless_steel_reduction (list): Stainless Steel Reduction Parameters
            additional_protection_reduction (list): Additional Protection Reduction
            allowance_deviation (list): Allowance Deviation Parameters
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Concrete Durabilities
        clientObject = model.clientModel.factory.create('ns0:concrete_durability')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Concrete Durability No.
        clientObject.no = no

        # User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Assigned Members
        clientObject.members = ConvertToDlString(members_no)

        # Assigned Member Sets
        clientObject.member_sets = ConvertToDlString(member_sets_no)

        # Exposure Classes for Reinforcement
        clientObject.no_risk_of_corrosion_or_attack_enabled = exposure_classes_reinforcement[0]
        clientObject.corrosion_induced_by_carbonation_enabled = exposure_classes_reinforcement[1]
        clientObject.corrosion_induced_by_chlorides_enabled = exposure_classes_reinforcement[2]
        clientObject.corrosion_induced_by_chlorides_from_sea_water_enabled = exposure_classes_reinforcement[3]

        if all(exposure_classes_reinforcement):
            raise ValueError("WARNING: At least one reinforcement exposure class must be selected.")

        for i in exposure_classes_reinforcement:
            if not isinstance(i, bool):
                raise ValueError('WARNING: The type of parameters should be bool. Kindly check list inputs for completeness and correctness.')

        if exposure_classes_reinforcement[0]:
            clientObject.no_risk_of_corrosion_or_attack = "VERY_DRY"
            for i in exposure_classes_reinforcement[1:]:
                if i:
                    raise ValueError('WARNING: If No Risk of Corrosion is True, other parameters cannot be selected. Kindly check list inputs for completeness and correctness.')

        if exposure_classes_reinforcement[1] :
            clientObject.corrosion_induced_by_carbonation = exposure_classes_reinforcement_types[0].name
        if exposure_classes_reinforcement[2]:
            clientObject.corrosion_induced_by_chlorides = exposure_classes_reinforcement_types[1].name
        if exposure_classes_reinforcement[3]:
            clientObject.corrosion_induced_by_chlorides_from_sea_water = exposure_classes_reinforcement_types[2].name

        # Exposure Classes for Concrete
        for i in exposure_classes_concrete:
            if not isinstance(i, bool):
                raise ValueError('WARNING: The type of parameters should be bool. Kindly check list inputs for completeness and correctness.')

        if exposure_classes_concrete[0]:
            clientObject.freeze_thaw_attack_enabled = True
            clientObject.freeze_thaw_attack = exposure_classes_concrete_types[0].name
        if exposure_classes_concrete[1]:
            clientObject.chemical_attack_enabled = True
            clientObject.chemical_attack = exposure_classes_concrete_types[1].name
        if exposure_classes_concrete[2]:
            clientObject.concrete_corrosion_induced_by_wear_enabled = True
            clientObject.concrete_corrosion_induced_by_wear = exposure_classes_concrete_types[2].name

        # Structural Class
        clientObject.structural_class_type = structural_class[0].name

        if structural_class[0].name == "STANDARD":
            for i in structural_class[1:]:
                if not isinstance(i, bool):
                    raise ValueError('WARNING: The type of last three parameters should be bool. Kindly check list inputs for completeness and correctness.')

            clientObject.increase_design_working_life_from_50_to_100_years_enabled = structural_class[1]
            clientObject.position_of_reinforcement_not_affected_by_construction_process_enabled = structural_class[2]
            clientObject.special_quality_control_of_production_enabled = structural_class[3]
            clientObject.air_entrainment_of_more_than_4_percent_enabled = structural_class[4]
        elif structural_class[0].name == "DEFINED":
            clientObject.userdefined_structural_class = structural_class[1].name

        # Stainless Steel Concrete Cover Reduction
        clientObject.stainless_steel_enabled = stainless_steel_reduction[0]

        if not isinstance(stainless_steel_reduction[0], bool):
            raise ValueError('WARNING: The type of the first parameter should be bool. Kindly check list inputs for completeness and correctness.')

        if stainless_steel_reduction[0]:
            clientObject.stainless_steel_type = stainless_steel_reduction[1].name

            if stainless_steel_reduction[1].name == "DEFINED":
                clientObject.stainless_steel_factor = stainless_steel_reduction[2]

        # Additional Protection Concrete Cover Reduction
        clientObject.additional_protection_enabled = additional_protection_reduction[0]

        if not isinstance(additional_protection_reduction[0], bool):
            raise ValueError('WARNING: The type of the first parameter should be bool. Kindly check list inputs for completeness and correctness.')

        if additional_protection_reduction[0]:
            clientObject.additional_protection_type = additional_protection_reduction[1].name

            if additional_protection_reduction[1].name == "DEFINED":
                clientObject.additional_protection_factor = additional_protection_reduction[2]

        # Allowance for Deviation
        clientObject.allowance_of_deviation_type = allowance_deviation[0].name

        if allowance_deviation[0].name == "STANDARD":
            clientObject.concrete_cast_enabled = allowance_deviation[1]

            if not isinstance(allowance_deviation[1], bool):
                raise ValueError('WARNING: The type of the second parameter should be bool. Kindly check list inputs for completeness and correctness.')

            if allowance_deviation[1]:
                clientObject.concrete_cast = allowance_deviation[2].name

        elif allowance_deviation[0].name == "DEFINED":
            clientObject.userdefined_allowance_of_deviation_factor = allowance_deviation[1]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Global Parameter to client model
        model.clientModel.service.set_concrete_durability(clientObject)
